/*******************************************************************************
 * Copyright (c) 2006, 2013 Oracle and/or its affiliates. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
 * which accompanies this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *     Oracle - initial API and implementation
 *
 ******************************************************************************/
package org.eclipse.persistence.tools.mapping.orm;

import java.util.List;
import org.eclipse.persistence.tools.mapping.ExternalForm;
import org.eclipse.persistence.tools.utility.TextRange;

/**
 * This interface and describes the configurable properties defined for a entity result in the ORM
 * xml. An entity result is used in a sql result set mapping to define how the raw, jdbc data
 * conforms to a targeted entity.
 * <p>
 * Provisional API: This interface is part of an interim API that is still under development and
 * expected to change significantly before reaching stability. It is available at this early stage
 * to solicit feedback from pioneering adopters on the understanding that any code that uses this
 * API will almost certainly be broken (repeatedly) as the API evolves.<p>
 *
 * @see javax.persistence.EntityResult
 * @see http://jcp.org/aboutJava/communityprocess/final/jsr220/index.html, persistence specificaton
 *
 * @version 2.6
 */
@SuppressWarnings("nls")
public interface ExternalEntityResult extends ExternalForm {

	/**
	 * The attribute name used to store and retrieve the discriminator-column property.
	 */
	String DISCRIMINATOR_COLUMN = "discriminator-column";

	/**
	 * The attribute name used to store and retrieve the entity-class property.
	 */
	String ENTITY_CLASS = "entity-class";

	/**
	 * The node name used to store and retrieve the element encapsulated by this external form.
	 */
	String ENTITY_RESULT = "entity-result";

	/**
	 * Adds a field result with the given name.
	 */
	ExternalFieldResult addFieldResult(String name);

	/**
	 * Returns a list of field result mappings.
	 */
	List<ExternalFieldResult> fieldResults();

	/**
	 * Returns the count of all the field result mappings.
	 */
	int fieldResultsSize();

	/**
	 * Returns the discriminator column name that identifies this result.
	 */
	String getDiscriminatorColumnName();

	/**
	 * Returns the {@link TextRange} for the discriminator column name that identifies this result.
	 */
	TextRange getDiscriminatorColumnNameTextRange();

	/**
	 * Returns the name of the targeted entity class.
	 */
	String getEntityClassName();

	/**
	 * Returns the {@link TextRange} for the name of the targeted entity class.
	 */
	TextRange getEntityClassNameTextRange();

	/**
	 * Returns the field result at the specified position.
	 */
	ExternalFieldResult getFieldResult(int index);

	/**
	 * Removes the field result at the given position.
	 */
	void removeFieldResult(int index);

	/**
	 * Set the discriminator column name that identifies this result.
	 */
	void setDiscriminatorColumnName(String name);

	/**
	 * Set the name of the targeted entity class.
	 */
	void setEntityClassName(String className);
}